@@ -172,7 +172,7 @@ class DynamoDbCoreAdapter:
             print(f"Re-using the already created dynamodb client for the default region")
         else:
             print(f"Initializing the {self}. For local development, make sure that you are connected to internet."
-                  f"\nOtherwise the framework will get stuck at initializing the {self}")
+                  f"\nOtherwise the DynamoDB client will get stuck at initializing the {self}")
 
             dynamodb_regions = Session().get_available_regions("dynamodb")
             if region_name in dynamodb_regions:
@@ -211,15 +211,27 @@ class DynamoDbCoreAdapter:
                 if e.__class__.__name__ != "ResourceInUseException":
                     raise Exception(f"Create table if not exists request failed: Exception of type {type(e).__name__} occurred {str(e)}")
 
-    def put_item_dict(self, item_dict: dict) -> Response:
+    def put_record(self, item_dict: dict) -> bool:
         try:
             table = self.dynamodb.Table(self.table_name)
             response = table.put_item(Item=item_dict)
-            return response
+            return True if response is not None else False
         except ResourceNotExistsError:
             raise Exception(f"DynamoDb table {self.table_name} doesn't exist. Failed to save attributes to DynamoDb table.")
         except Exception as e:
-            raise Exception(f"Failed to save attributes to DynamoDb table. Exception of type {type(e).__name__} occurred: {str(e)}")
+            print(f"Failed to save attributes to DynamoDb table. Exception of type {type(e).__name__} occurred: {str(e)}")
+        return False
+
+    def delete_record(self, indexes_keys_selectors: dict) -> bool:
+        try:
+            table = self.dynamodb.Table(self.table_name)
+            response = table.delete_item(Key=indexes_keys_selectors)
+            return True if response is not None else False
+        except ResourceNotExistsError:
+            raise Exception(f"DynamoDb table {self.table_name} doesn't exist. Failed to save attributes to DynamoDb table.")
+        except Exception as e:
+            print(e)
+        return False
 
     def get_item_by_primary_key(self, key_name: str, key_value: any, fields_to_get: Optional[List[str]]) -> Optional[GetItemResponse]:
         if fields_to_get is not None:
@@ -233,15 +245,16 @@ class DynamoDbCoreAdapter:
             table = self.dynamodb.Table(self.table_name)
             response = table.get_item(**kwargs)
             if "Item" in response:
-                return GetItemResponse(item=Utils.dynamodb_to_python(dynamodb_object=response["Item"]), success=True)
+                e = Utils.dynamodb_to_python_higher_level(dynamodb_object=response["Item"])
+                return GetItemResponse(item=Utils.dynamodb_to_python_higher_level(dynamodb_object=response["Item"]), success=True)
             else:
                 return GetItemResponse(item=None, success=False)
         except ResourceNotExistsError:
             raise Exception(f"DynamoDb table {self.table_name} do not exist or in the process"
                             "of being created. Failed to get attributes from DynamoDb table.")
         except Exception as e:
-            raise Exception(f"Failed to retrieve attributes from DynamoDb table."
-                            f"Exception of type {type(e).__name__} occurred: {str(e)}")
+            print(f"Failed to retrieve attributes from DynamoDb table. Exception of type {type(e).__name__} occurred: {str(e)}")
+            return None
 
     def _execute_update_query(self, query_kwargs_dict: dict) -> Optional[Response]:
         try:
@@ -255,9 +268,8 @@ class DynamoDbCoreAdapter:
             print(f"{e} - No element has been found for the update query : {query_kwargs_dict}")
             return None
         except Exception as e:
-            print(type(e))
-            raise Exception(f"Failed to update attributes in DynamoDb table."
-                            f"Exception of type {type(e).__name__} occurred: {str(e)}")
+            print(f"Failed to update attributes in DynamoDb table. Exception of type {type(e).__name__} occurred: {str(e)}")
+            return None
 
     def add_data_elements_to_list(self, key_name: str, key_value: Any, object_path: str,
                                   element_values: List[dict]) -> Optional[Response]:
@@ -291,6 +303,33 @@ class DynamoDbCoreAdapter:
 
         return self._execute_update_query(query_kwargs_dict=kwargs)
 
+    def remove_data_elements_from_map(self, key_name: str, key_value: Any,
+                                      targets_path_elements: List[List[DatabasePathElement]]) -> Optional[Response]:
+        expression_attribute_names_dict = dict()
+        update_expression = "REMOVE "
+
+        num_targets = len(targets_path_elements)
+        for i_target, target in enumerate(targets_path_elements):
+            current_target_num_path_elements = len(target)
+            for i_path_element, path_element in enumerate(target):
+                current_path_key = f"#target{i_target}_pathKey{i_path_element}"
+                update_expression += current_path_key
+                expression_attribute_names_dict[current_path_key] = path_element.element_key
+                if i_path_element + 1 < current_target_num_path_elements:
+                    update_expression += "."
+            if i_target + 1 < num_targets:
+                update_expression += ", "
+
+        update_query_kwargs = {
+            "TableName": self.table_name,
+            "Key": {key_name: key_value},
+            "ReturnValues": "UPDATED_NEW",
+            "UpdateExpression": update_expression,
+            "ExpressionAttributeNames": expression_attribute_names_dict,
+        }
+        response = self._execute_update_query(query_kwargs_dict=update_query_kwargs)
+        return response
+
     def initialize_all_elements_in_map_target(self, key_name: str, key_value: Any, target_path_elements: List[DatabasePathElement]) -> bool:
         current_path_target = ""
         expression_attribute_names_dict: Dict[str, str] = dict()
@@ -318,8 +357,8 @@ class DynamoDbCoreAdapter:
                 return False
         return True
 
-    def set_update_data_element_to_map(self, key_name: str, key_value: Any,
-                                       target_path_elements: List[DatabasePathElement], value: Any) -> Optional[Response]:
+    def set_update_data_element_to_map(self, key_name: str, key_value: Any, value: Any,
+                                       target_path_elements: List[DatabasePathElement]) -> Optional[Response]:
         expression_attribute_names_dict = dict()
         update_expression = "SET "
 
