@@ -1,8 +1,8 @@
 from typing import Optional, List, Dict, Any
 from StructNoSQL.dynamodb.dynamodb_core import DynamoDbCoreAdapter, PrimaryIndex, GlobalSecondaryIndex, \
-    DynamoDBMapObjectSetter
-from StructNoSQL.dynamodb.models import DatabasePathElement, FieldGetter, FieldSetter
-from StructNoSQL.fields import BaseField, MapModel, MapField, MapItem
+    DynamoDBMapObjectSetter, Response
+from StructNoSQL.dynamodb.models import DatabasePathElement, FieldGetter, FieldSetter, FieldRemover
+from StructNoSQL.fields import BaseField, MapModel, MapField, MapItem, TableDataModel
 from StructNoSQL.practical_logger import message_with_vars
 from StructNoSQL.utils.process_render_fields_paths import process_and_get_fields_paths_objects_from_fields_paths, \
     process_and_make_single_rendered_database_path, process_validate_data_and_make_single_rendered_database_path
@@ -27,10 +27,50 @@ class BaseTable:
         )
 
         if not isinstance(data_model, type):
-            self.model = data_model
+            self._model = data_model
         else:
-            self.model = data_model()
-        assign_internal_mapping_from_class(table=self, class_instance=self.model)
+            self._model = data_model()
+        self._model_virtual_map_field = None
+
+        assign_internal_mapping_from_class(table=self, class_instance=self._model)
+
+    @property
+    def model(self) -> TableDataModel:
+        return self._model
+
+    @property
+    def model_virtual_map_field(self) -> MapField:
+        if self._model_virtual_map_field is None:
+            self._model_virtual_map_field = MapField(name="", model=self._model)
+            # The model_virtual_map_field is a MapField with no name, that use the table model, which easily
+            # give us the ability to use the functions of the MapField object (for example, functions for
+            # data validation), with the data model of the table itself. For example, the put_record
+            # operation, needs to validate its data, based on the table data model, not a MapField.
+        return self._model_virtual_map_field
+
+    def put_record(self, record_dict_data: dict) -> bool:
+        self.model_virtual_map_field.populate(value=record_dict_data)
+        validated_data, is_valid = self.model_virtual_map_field.validate_data()
+        if is_valid is True:
+            return self.dynamodb_client.put_record(item_dict=validated_data)
+        else:
+            return False
+
+    def delete_record(self, indexes_keys_selectors: dict) -> bool:
+        found_all_indexes = True
+        for index_key, index_target_value in indexes_keys_selectors.items():
+            index_matching_field = getattr(self.model, index_key, None)
+            if index_matching_field is None:
+                found_all_indexes = False
+                print(message_with_vars(
+                    message="An index key selector passed to the delete_record function, was not found, in the table model. Operation not executed.",
+                    vars_dict={"index_key": index_key, "index_target_value": index_target_value, "index_matching_field": index_matching_field, "table.model": self.model}
+                ))
+
+        if found_all_indexes is True:
+            return self.dynamodb_client.delete_record(indexes_keys_selectors=indexes_keys_selectors)
+        else:
+            return False
 
 
     def get_single_field_item_from_single_item(self, key_name: str, key_value: str, field_to_get: str, query_kwargs: Optional[dict] = None) -> Any:
@@ -130,6 +170,38 @@ class BaseTable:
         return True if response is not None else False
 
 
+    def remove_single_item_at_path_target(self, key_name: str, key_value: str, target_field: str,
+                                          query_kwargs: Optional[dict] = None) -> bool:
+        response: Optional[Response] = self.dynamodb_client.remove_data_elements_from_map(
+            key_name=key_name, key_value=key_value,
+            targets_path_elements=[process_and_make_single_rendered_database_path(
+                field_path=target_field, fields_switch=self.fields_switch, query_kwargs=query_kwargs
+            )]
+        )
+        return True if response is not None else False
+
+    def remove_multiple_items_at_path_targets(self, key_name: str, key_value: str, removers: List[FieldRemover]) -> bool:
+        if len(removers) > 0:
+            removers_database_paths: List[List[DatabasePathElement]] = list()
+            for current_remover in removers:
+                removers_database_paths.append(
+                    process_and_make_single_rendered_database_path(
+                        field_path=current_remover.target_path,
+                        fields_switch=self.fields_switch,
+                        query_kwargs=current_remover.query_kwargs
+                    )
+                )
+
+            response: Optional[Response] = self.dynamodb_client.remove_data_elements_from_map(
+                key_name=key_name, key_value=key_value, targets_path_elements=removers_database_paths
+            )
+            return True if response is not None else False
+        else:
+            # If no remover has been specified, we do not run the database operation, yet we still
+            # return True, since technically, what needed to be performed (nothing) was performed.
+            return True
+
+
     @property
     def internal_mapping(self) -> dict:
         return self._internal_mapping
@@ -168,8 +240,7 @@ def assign_internal_mapping_from_class(table: BaseTable, class_instance: Optiona
             ))
 
     class_variables = class_type.__dict__
-    num_required_fields: Optional[list] = getattr(class_type, "_num_required_fields", None)
-    # A static variable of a type is not accessible in the __dict__, but it is in the attributes.
+    required_fields = list()
 
     for variable_key, variable_item in class_variables.items():
         current_field_path = "" if nested_field_path is None else f"{nested_field_path}"
@@ -192,9 +263,8 @@ def assign_internal_mapping_from_class(table: BaseTable, class_instance: Optiona
                 variable_item._table = table
                 output_mapping[variable_key] = ""
 
-                if num_required_fields is not None:
-                    if variable_item.required is True:
-                        num_required_fields += 1
+                if variable_item.required is True:
+                    required_fields.append(variable_item)
 
                 current_field_path += f"{variable_item.field_name}" if len(current_field_path) == 0 else f".{variable_item.field_name}"
                 table.fields_switch[current_field_path] = variable_item
@@ -223,37 +293,22 @@ def assign_internal_mapping_from_class(table: BaseTable, class_instance: Optiona
                 variable_item._database_path = [*current_path_elements, new_database_path_element]
                 variable_item._table = table
 
-                if num_required_fields is not None:
-                    if variable_item.required is True:
-                        num_required_fields += 1
+                if variable_item.required is True:
+                    required_fields.append(variable_item)
 
                 current_field_path += f"{variable_item.field_name}" if len(current_field_path) == 0 else f".{variable_item.field_name}"
                 table.fields_switch[current_field_path] = variable_item
 
                 output_mapping[variable_item.field_name] = assign_internal_mapping_from_class(
-                    table=table, class_type=variable_item,
-                    nested_field_path=current_field_path,
+                    table=table, class_type=variable_item, nested_field_path=current_field_path,
                     current_path_elements=variable_item.database_path
                 )
 
-            elif MapModel in variable_bases:
-                variable_item: MapField
-                print(variable_item)
-                continue
-
-                variable_item: MapModel
-                variable_item._database_path = {**current_path_elements}
-                output_mapping[variable_key] = assign_internal_mapping_from_class(
-                    table=table, class_type=variable_item, current_path_elements=variable_item.database_path
-                )
-
-
         except Exception as e:
             print(e)
 
-    if num_required_fields is not None:
-        setattr(class_type, "_num_required_fields", num_required_fields)
-        # We need to set the attribute, because when we go the num_required_fields with the get_attr
-        # function, we did not get a reference to the attribute, but a copy of the attribute value.
+    setattr(class_type, "required_fields", required_fields)
+    # We need to set the attribute, because when we go the required_fields with the get_attr
+    # function, we did not get a reference to the attribute, but a copy of the attribute value.
 
     return output_mapping
